@@ -243,7 +243,7 @@ TEST_P(TlsCipherSuiteTest, SingleCipherSuite) {
 
 TEST_P(TlsCipherSuiteTest, ResumeCipherSuite) {
   if (SkipIfCipherSuiteIsDSA()) {
-    return;  // Tickets don't work with DSA (bug 1174677).
+    GTEST_SKIP() << "Tickets not supported with DSA (bug 1174677).";
   }
 
   SetupCertificate();  // This is only needed once.
@@ -330,7 +330,7 @@ TEST_P(TlsCipherSuiteTest, ReadLimit) {
 TEST_P(TlsCipherSuiteTest, WriteLimit) {
   // This asserts in TLS 1.3 because we expect an automatic update.
   if (version_ >= SSL_LIBRARY_VERSION_TLS_1_3) {
-    return;
+    GTEST_SKIP();
   }
   SetupCertificate();
   EnableSingleCipher();
@@ -348,7 +348,7 @@ TEST_P(TlsCipherSuiteTest, WriteLimit) {
   static const uint16_t k##name##CiphersArr[] = {__VA_ARGS__};                 \
   static const ::testing::internal::ParamGenerator<uint16_t>                   \
       k##name##Ciphers = ::testing::ValuesIn(k##name##CiphersArr);             \
-  INSTANTIATE_TEST_CASE_P(                                                     \
+  INSTANTIATE_TEST_SUITE_P(                                                    \
       CipherSuite##name, TlsCipherSuiteTest,                                   \
       ::testing::Combine(TlsConnectTestBase::kTlsVariants##modes,              \
                          TlsConnectTestBase::kTls##versions, k##name##Ciphers, \
@@ -525,7 +525,7 @@ static const SecStatusParams kSecStatusTestValuesArr[] = {
      "AES-256-GCM", 256},
     {SSL_LIBRARY_VERSION_TLS_1_2, TLS_ECDHE_RSA_WITH_CHACHA20_POLY1305_SHA256,
      "ChaCha20-Poly1305", 256}};
-INSTANTIATE_TEST_CASE_P(TestSecurityStatus, SecurityStatusTest,
-                        ::testing::ValuesIn(kSecStatusTestValuesArr));
+INSTANTIATE_TEST_SUITE_P(TestSecurityStatus, SecurityStatusTest,
+                         ::testing::ValuesIn(kSecStatusTestValuesArr));
 
 }  // namespace nss_test
