@@ -36,7 +36,7 @@ pk11_setAttributes(PK11SlotInfo *slot, CK_OBJECT_HANDLE id,
     CK_SESSION_HANDLE rwsession;
 
     rwsession = PK11_GetRWSession(slot);
-    if (rwsession == CK_INVALID_SESSION) {
+    if (rwsession == CK_INVALID_HANDLE) {
         PORT_SetError(SEC_ERROR_BAD_DATA);
         return SECFailure;
     }
@@ -74,7 +74,7 @@ pk11_copyAttributes(PLArenaPool *arena,
             return SECFailure;
         }
         /* remove the unknown attributes. If we don't have enough attributes
-	 * PK11_CreateNewObject() will fail */
+         * PK11_CreateNewObject() will fail */
         for (i = 0, j = 0; i < copyTemplateCount; i++) {
             if (copyTemplate[i].ulValueLen != -1) {
                 newTemplate[j] = copyTemplate[i];
@@ -88,11 +88,12 @@ pk11_copyAttributes(PLArenaPool *arena,
     }
     if (crv != CKR_OK) {
         PORT_SetError(PK11_MapError(crv));
+        PORT_Free(newTemplate);
         return SECFailure;
     }
     if (targetID == CK_INVALID_HANDLE) {
         /* we need to create the object */
-        rv = PK11_CreateNewObject(targetSlot, CK_INVALID_SESSION,
+        rv = PK11_CreateNewObject(targetSlot, CK_INVALID_HANDLE,
                                   copyTemplate, copyTemplateCount, PR_TRUE, &targetID);
     } else {
         /* update the existing object with the new attributes */
@@ -100,7 +101,7 @@ pk11_copyAttributes(PLArenaPool *arena,
                                 copyTemplate, copyTemplateCount);
     }
     if (newTemplate) {
-        free(newTemplate);
+        PORT_Free(newTemplate);
     }
     return rv;
 }
@@ -463,7 +464,7 @@ pk11_mergeSecretKey(PK11SlotInfo *targetSlot, PK11SlotInfo *sourceSlot,
         { CKA_ID, NULL, 0 },
         { CKA_CLASS, NULL, 0 }
     };
-    CK_ULONG symTemplateCount = sizeof(symTemplate) / sizeof(symTemplate[0]);
+    const CK_ULONG symTemplateCount = sizeof(symTemplate) / sizeof(symTemplate[0]);
     CK_ATTRIBUTE symCopyTemplate[] = {
         { CKA_LABEL, NULL, 0 }
     };
