@@ -24,9 +24,11 @@
  *     uint8 indicator = 0xff;            // To disambiguate from tickets.
  *     uint16 cipherSuite;                // Selected cipher suite.
  *     uint16 keyShare;                   // Requested key share group (0=none)
+ *     PRUint8 echConfigId;               // ECH config_id
+ *     HpkeKdfId kdfId;                   // ECH KDF (uint16)
+ *     HpkeAeadId aeadId;                 // ECH AEAD (uint16)
+ *     opaque echHpkeCtx<0..65535>;       // ECH serialized HPKE context
  *     opaque applicationToken<0..65535>; // Application token
- *     echConfigId<0..255>;               // Encrypted Client Hello config_id
- *     echHrrPsk<0..255>;                 // Encrypted Client Hello HRR PSK
  *     opaque ch_hash[rest_of_buffer];    // H(ClientHello)
  * } CookieInner;
  *
@@ -43,10 +45,7 @@ tls13_MakeHrrCookie(sslSocket *ss, const sslNamedGroupDef *selectedGroup,
     PRUint8 cookie[1024];
     sslBuffer cookieBuf = SSL_BUFFER(cookie);
     static const PRUint8 indicator = 0xff;
-    SECItem hrrNonceInfoItem = { siBuffer, (unsigned char *)kHpkeInfoEchHrr,
-                                 strlen(kHpkeInfoEchHrr) };
-    PK11SymKey *echHrrPsk = NULL;
-    SECItem *rawEchPsk = NULL;
+    SECItem *echHpkeCtx = NULL;
 
     /* Encode header. */
     rv = sslBuffer_Append(&cookieBuf, &indicator, 1);
@@ -63,54 +62,56 @@ tls13_MakeHrrCookie(sslSocket *ss, const sslNamedGroupDef *selectedGroup,
         return SECFailure;
     }
 
+    if (ss->xtnData.ech) {
+        /* Record that we received ECH. */
+        rv = sslBuffer_AppendNumber(&cookieBuf, PR_TRUE, 1);
+        if (rv != SECSuccess) {
+            return SECFailure;
+        }
+
+        rv = sslBuffer_AppendNumber(&cookieBuf, ss->xtnData.ech->configId,
+                                    1);
+        if (rv != SECSuccess) {
+            return SECFailure;
+        }
+
+        rv = sslBuffer_AppendNumber(&cookieBuf, ss->xtnData.ech->kdfId, 2);
+        if (rv != SECSuccess) {
+            return SECFailure;
+        }
+        rv = sslBuffer_AppendNumber(&cookieBuf, ss->xtnData.ech->aeadId, 2);
+        if (rv != SECSuccess) {
+            return SECFailure;
+        }
+
+        /* There might be no HPKE Context, e.g. when we lack a matching ECHConfig. */
+        if (ss->ssl3.hs.echHpkeCtx) {
+            rv = PK11_HPKE_ExportContext(ss->ssl3.hs.echHpkeCtx, NULL, &echHpkeCtx);
+            if (rv != SECSuccess) {
+                return SECFailure;
+            }
+            rv = sslBuffer_AppendVariable(&cookieBuf, echHpkeCtx->data, echHpkeCtx->len, 2);
+            SECITEM_ZfreeItem(echHpkeCtx, PR_TRUE);
+        } else {
+            /* Zero length HPKE context. */
+            rv = sslBuffer_AppendNumber(&cookieBuf, 0, 2);
+        }
+        if (rv != SECSuccess) {
+            return SECFailure;
+        }
+    } else {
+        rv = sslBuffer_AppendNumber(&cookieBuf, PR_FALSE, 1);
+        if (rv != SECSuccess) {
+            return SECFailure;
+        }
+    }
+
     /* Application token. */
     rv = sslBuffer_AppendVariable(&cookieBuf, appToken, appTokenLen, 2);
     if (rv != SECSuccess) {
         return SECFailure;
     }
 
-    /* Received ECH config_id, regardless of acceptance or possession
-     * of a matching ECHConfig. If rejecting ECH, this is essentially a boolean
-     * indicating that ECH was offered in CH1. If accepting ECH, this config_id
-     * will be used for the ECH decryption in CH2. */
-    if (ss->xtnData.echConfigId.len) {
-        rv = sslBuffer_AppendVariable(&cookieBuf, ss->xtnData.echConfigId.data,
-                                      ss->xtnData.echConfigId.len, 1);
-    } else {
-        PORT_Assert(!ssl3_FindExtension(ss, ssl_tls13_encrypted_client_hello_xtn));
-        rv = sslBuffer_AppendNumber(&cookieBuf, 0, 1);
-    }
-    if (rv != SECSuccess) {
-        return SECFailure;
-    }
-
-    /* Extract and encode the ech-hrr-key, if ECH was accepted
-     * (i.e. an Open() succeeded. */
-    if (ss->ssl3.hs.echAccepted) {
-        rv = PK11_HPKE_ExportSecret(ss->ssl3.hs.echHpkeCtx, &hrrNonceInfoItem, 32, &echHrrPsk);
-        if (rv != SECSuccess) {
-            return SECFailure;
-        }
-        rv = PK11_ExtractKeyValue(echHrrPsk);
-        if (rv != SECSuccess) {
-            PK11_FreeSymKey(echHrrPsk);
-            return SECFailure;
-        }
-        rawEchPsk = PK11_GetKeyData(echHrrPsk);
-        if (!rawEchPsk) {
-            PK11_FreeSymKey(echHrrPsk);
-            return SECFailure;
-        }
-        rv = sslBuffer_AppendVariable(&cookieBuf, rawEchPsk->data, rawEchPsk->len, 1);
-        PK11_FreeSymKey(echHrrPsk);
-    } else {
-        /* Zero length ech_hrr_key. */
-        rv = sslBuffer_AppendNumber(&cookieBuf, 0, 1);
-    }
-    if (rv != SECSuccess) {
-        return SECFailure;
-    }
-
     /* Compute and encode hashes. */
     rv = tls13_ComputeHandshakeHashes(ss, &hashes);
     if (rv != SECSuccess) {
@@ -131,23 +132,38 @@ tls13_MakeHrrCookie(sslSocket *ss, const sslNamedGroupDef *selectedGroup,
     return SECSuccess;
 }
 
-/* Recover the hash state from the cookie. */
+/* Given a cookie and cookieLen, decrypt and parse, returning
+ * any values that were requested via the "previous_" params. If
+ * recoverState is true, the transcript state and application
+ * token are restored. Note that previousEchKdfId, previousEchAeadId,
+ * previousEchConfigId, and previousEchHpkeCtx are not modified if ECH was not
+ * previously negotiated (i.e., previousEchOffered is PR_FALSE). */
 SECStatus
-tls13_RecoverHashState(sslSocket *ss,
-                       unsigned char *cookie, unsigned int cookieLen,
-                       ssl3CipherSuite *previousCipherSuite,
-                       const sslNamedGroupDef **previousGroup,
-                       PRBool *previousEchOffered)
+tls13_HandleHrrCookie(sslSocket *ss,
+                      unsigned char *cookie, unsigned int cookieLen,
+                      ssl3CipherSuite *previousCipherSuite,
+                      const sslNamedGroupDef **previousGroup,
+                      PRBool *previousEchOffered,
+                      HpkeKdfId *previousEchKdfId,
+                      HpkeAeadId *previousEchAeadId,
+                      PRUint8 *previousEchConfigId,
+                      HpkeContext **previousEchHpkeCtx,
+                      PRBool recoverState)
 {
     SECStatus rv;
     unsigned char plaintext[1024];
     unsigned int plaintextLen = 0;
     sslBuffer messageBuf = SSL_BUFFER_EMPTY;
-    sslReadBuffer echPskBuf;
-    sslReadBuffer echConfigIdBuf;
+    sslReadBuffer echHpkeBuf = { 0 };
+    PRBool receivedEch;
+    PRUint8 echConfigId = 0;
     PRUint64 sentinel;
     PRUint64 cipherSuite;
+    HpkeContext *hpkeContext = NULL;
+    HpkeKdfId echKdfId = 0;
+    HpkeAeadId echAeadId = 0;
     PRUint64 group;
+    PRUint64 tmp64;
     const sslNamedGroupDef *selectedGroup;
     PRUint64 appTokenLen;
 
@@ -180,19 +196,62 @@ tls13_RecoverHashState(sslSocket *ss,
     }
     selectedGroup = ssl_LookupNamedGroup(group);
 
+    /* Was ECH received. */
+    rv = sslRead_ReadNumber(&reader, 1, &tmp64);
+    if (rv != SECSuccess) {
+        FATAL_ERROR(ss, SSL_ERROR_RX_MALFORMED_CLIENT_HELLO, illegal_parameter);
+        return SECFailure;
+    }
+    receivedEch = tmp64 == PR_TRUE;
+
+    if (receivedEch) {
+        /* ECH config ID */
+        rv = sslRead_ReadNumber(&reader, 1, &tmp64);
+        if (rv != SECSuccess) {
+            FATAL_ERROR(ss, SSL_ERROR_RX_MALFORMED_CLIENT_HELLO, illegal_parameter);
+            return SECFailure;
+        }
+        echConfigId = tmp64;
+
+        /* ECH Ciphersuite */
+        rv = sslRead_ReadNumber(&reader, 2, &tmp64);
+        if (rv != SECSuccess) {
+            FATAL_ERROR(ss, SSL_ERROR_RX_MALFORMED_CLIENT_HELLO, illegal_parameter);
+            return SECFailure;
+        }
+        echKdfId = (HpkeKdfId)tmp64;
+
+        rv = sslRead_ReadNumber(&reader, 2, &tmp64);
+        if (rv != SECSuccess) {
+            FATAL_ERROR(ss, SSL_ERROR_RX_MALFORMED_CLIENT_HELLO, illegal_parameter);
+            return SECFailure;
+        }
+        echAeadId = (HpkeAeadId)tmp64;
+
+        /* ECH HPKE context may be empty. */
+        rv = sslRead_ReadVariable(&reader, 2, &echHpkeBuf);
+        if (rv != SECSuccess) {
+            FATAL_ERROR(ss, SSL_ERROR_RX_MALFORMED_CLIENT_HELLO, illegal_parameter);
+            return SECFailure;
+        }
+
+        if (previousEchHpkeCtx && echHpkeBuf.len) {
+            const SECItem hpkeItem = { siBuffer, CONST_CAST(unsigned char, echHpkeBuf.buf),
+                                       echHpkeBuf.len };
+            hpkeContext = PK11_HPKE_ImportContext(&hpkeItem, NULL);
+            if (!hpkeContext) {
+                FATAL_ERROR(ss, PORT_GetError(), illegal_parameter);
+                return SECFailure;
+            }
+        }
+    }
+
     /* Application token. */
-    PORT_Assert(ss->xtnData.applicationToken.len == 0);
     rv = sslRead_ReadNumber(&reader, 2, &appTokenLen);
     if (rv != SECSuccess) {
         FATAL_ERROR(ss, SSL_ERROR_RX_MALFORMED_CLIENT_HELLO, illegal_parameter);
         return SECFailure;
     }
-    if (SECITEM_AllocItem(NULL, &ss->xtnData.applicationToken,
-                          appTokenLen) == NULL) {
-        FATAL_ERROR(ss, PORT_GetError(), internal_error);
-        return SECFailure;
-    }
-    ss->xtnData.applicationToken.len = appTokenLen;
     sslReadBuffer appTokenReader = { 0 };
     rv = sslRead_Read(&reader, appTokenLen, &appTokenReader);
     if (rv != SECSuccess) {
@@ -200,57 +259,74 @@ tls13_RecoverHashState(sslSocket *ss,
         return SECFailure;
     }
     PORT_Assert(appTokenReader.len == appTokenLen);
-    PORT_Memcpy(ss->xtnData.applicationToken.data, appTokenReader.buf, appTokenLen);
 
-    /* ECH Config ID, which may be empty. */
-    rv = sslRead_ReadVariable(&reader, 1, &echConfigIdBuf);
-    if (rv != SECSuccess) {
-        FATAL_ERROR(ss, SSL_ERROR_RX_MALFORMED_CLIENT_HELLO, illegal_parameter);
-        return SECFailure;
-    }
-    /* ECH HRR PSK, if present, is already used by tls13_GetEchInfoFromCookie */
-    rv = sslRead_ReadVariable(&reader, 1, &echPskBuf);
-    if (rv != SECSuccess) {
-        FATAL_ERROR(ss, SSL_ERROR_RX_MALFORMED_CLIENT_HELLO, illegal_parameter);
-        return SECFailure;
+    if (recoverState) {
+        PORT_Assert(ss->xtnData.applicationToken.len == 0);
+        if (SECITEM_AllocItem(NULL, &ss->xtnData.applicationToken,
+                              appTokenLen) == NULL) {
+            FATAL_ERROR(ss, PORT_GetError(), internal_error);
+            return SECFailure;
+        }
+        PORT_Memcpy(ss->xtnData.applicationToken.data, appTokenReader.buf, appTokenLen);
+        ss->xtnData.applicationToken.len = appTokenLen;
+
+        /* The remainder is the hash. */
+        unsigned int hashLen = SSL_READER_REMAINING(&reader);
+        if (hashLen != tls13_GetHashSize(ss)) {
+            FATAL_ERROR(ss, SSL_ERROR_RX_MALFORMED_CLIENT_HELLO, illegal_parameter);
+            return SECFailure;
+        }
+
+        /* Now reinject the message. */
+        SSL_ASSERT_HASHES_EMPTY(ss);
+        rv = ssl_HashHandshakeMessageInt(ss, ssl_hs_message_hash, 0,
+                                         SSL_READER_CURRENT(&reader), hashLen,
+                                         ssl3_UpdateHandshakeHashes);
+        if (rv != SECSuccess) {
+            return SECFailure;
+        }
+
+        /* And finally reinject the HRR. */
+        rv = tls13_ConstructHelloRetryRequest(ss, cipherSuite,
+                                              selectedGroup,
+                                              cookie, cookieLen,
+                                              &messageBuf);
+        if (rv != SECSuccess) {
+            return SECFailure;
+        }
+
+        rv = ssl_HashHandshakeMessageInt(ss, ssl_hs_server_hello, 0,
+                                         SSL_BUFFER_BASE(&messageBuf),
+                                         SSL_BUFFER_LEN(&messageBuf),
+                                         ssl3_UpdateHandshakeHashes);
+        sslBuffer_Clear(&messageBuf);
+        if (rv != SECSuccess) {
+            return SECFailure;
+        }
     }
 
-    /* The remainder is the hash. */
-    unsigned int hashLen = SSL_READER_REMAINING(&reader);
-    if (hashLen != tls13_GetHashSize(ss)) {
-        FATAL_ERROR(ss, SSL_ERROR_RX_MALFORMED_CLIENT_HELLO, illegal_parameter);
-        return SECFailure;
+    if (previousCipherSuite) {
+        *previousCipherSuite = cipherSuite;
     }
-
-    /* Now reinject the message. */
-    SSL_ASSERT_HASHES_EMPTY(ss);
-    rv = ssl_HashHandshakeMessageInt(ss, ssl_hs_message_hash, 0,
-                                     SSL_READER_CURRENT(&reader), hashLen,
-                                     ssl3_UpdateHandshakeHashes);
-    if (rv != SECSuccess) {
-        return SECFailure;
+    if (previousGroup) {
+        *previousGroup = selectedGroup;
     }
-
-    /* And finally reinject the HRR. */
-    rv = tls13_ConstructHelloRetryRequest(ss, cipherSuite,
-                                          selectedGroup,
-                                          cookie, cookieLen,
-                                          &messageBuf);
-    if (rv != SECSuccess) {
-        return SECFailure;
+    if (previousEchOffered) {
+        *previousEchOffered = receivedEch;
     }
-
-    rv = ssl_HashHandshakeMessageInt(ss, ssl_hs_server_hello, 0,
-                                     SSL_BUFFER_BASE(&messageBuf),
-                                     SSL_BUFFER_LEN(&messageBuf),
-                                     ssl3_UpdateHandshakeHashes);
-    sslBuffer_Clear(&messageBuf);
-    if (rv != SECSuccess) {
-        return SECFailure;
+    if (receivedEch) {
+        if (previousEchConfigId) {
+            *previousEchConfigId = echConfigId;
+        }
+        if (previousEchKdfId) {
+            *previousEchKdfId = echKdfId;
+        }
+        if (previousEchAeadId) {
+            *previousEchAeadId = echAeadId;
+        }
+        if (previousEchHpkeCtx) {
+            *previousEchHpkeCtx = hpkeContext;
+        }
     }
-
-    *previousCipherSuite = cipherSuite;
-    *previousGroup = selectedGroup;
-    *previousEchOffered = echConfigIdBuf.len > 0;
     return SECSuccess;
 }
